define(["require", "exports", "./../index", "deep-equal", "deepcopy"], function (require, exports, index_1, deepEqual, copy) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Property {
        constructor(schema, datastoreServiceFactory, context, onInitCallback) {
            this._schema = undefined;
            this._datastoreServiceFactory = undefined;
            this._datastoreService = undefined;
            this._context = undefined;
            this._onInitCallback = undefined;
            this._remoteValue = undefined;
            this._value = undefined;
            this._schema = schema;
            this._datastoreServiceFactory = datastoreServiceFactory;
            this._context = context;
            this._onInitCallback = onInitCallback;
        }
        get schema() {
            return this._schema;
        }
        async getDatastoreService() {
            if (!this._datastoreService) {
                this._datastoreService = await this._datastoreServiceFactory();
            }
            return this._datastoreService;
        }
        get context() {
            return this._context;
        }
        get onInitCallback() {
            return this._onInitCallback;
        }
        get remoteValue() {
            return this._remoteValue;
        }
        set remoteValue(value) {
            this._remoteValue = value;
        }
        get value() {
            return this._value;
        }
        set value(value) {
            this._value = value;
        }
        isDirty() {
            return !deepEqual(this.value, this.remoteValue);
        }
        isEntity() {
            return this._isEntity(this.value);
        }
        isEntityReference() {
            return this._isEntityReference(this.value);
        }
        isEntityCollectionReference() {
            return this._isEntityCollectionReference(this.value);
        }
        async initializeValue() {
            if (this.onInitCallback) {
                const initialValue = await this.onInitCallback();
                this.value = initialValue;
            }
        }
        getValue(valueSource) {
            return this._getCurrentValue();
        }
        _buildEntityCollectionByReferenceQuery(currentValue, options) {
            const baseQuery = {
                attribute: currentValue.relatedAttribute,
                operator: '=',
                value: currentValue.id,
            };
            const queryOptions = (options || {});
            if (queryOptions.query) {
                queryOptions.query = {
                    operator: 'and',
                    queries: [
                        queryOptions.query,
                        baseQuery,
                    ],
                };
            }
            else {
                queryOptions.query = baseQuery;
            }
            return queryOptions;
        }
        async getValueLazy(valueSource, context, options) {
            const currentValue = this._getCurrentValue();
            const isEntityReference = this._isEntityReference(currentValue);
            const isEntityCollectionReference = this._isEntityCollectionReference(currentValue);
            if (currentValue && currentValue.constructor && currentValue.constructor.name === 'EntityCollection') {
                return currentValue;
            }
            if (isEntityCollectionReference) {
                const queryOptions = this._buildEntityCollectionByReferenceQuery(currentValue, options);
                const coll = await this._getEntityCollectionByReference(currentValue.relatedType, context, queryOptions);
                this.remoteValue = this.value;
                this.value = coll;
                return coll;
            }
            if (!isEntityReference) {
                return currentValue;
            }
            let privateGetOptions;
            if (isEntityReference) {
                const entityType = await (await this.getDatastoreService()).getEntityType(currentValue.type);
                privateGetOptions = {};
                privateGetOptions.expandEntity = entityType.getDefaultExpandOptions(index_1.ExpandType.Entity);
            }
            else {
                privateGetOptions = currentValue._options || {};
            }
            const entity = await this._getEntityByReference(currentValue, context, privateGetOptions);
            this.remoteValue = this.value;
            this.value = entity;
            return entity;
        }
        setValue(valueTarget, value) {
            this._value = value;
        }
        updateFromRemote(remoteValue) {
            this._remoteValue = remoteValue;
            this._value = copy(remoteValue);
        }
        _getCurrentValue() {
            return this.value === undefined ? this.remoteValue : this.value;
        }
        _isEntity(value) {
            return value.getEntityReference !== undefined && value.entityType !== undefined && value.propertyBag !== undefined;
        }
        _isEntityReference(value) {
            return value && value.constructor && value.constructor.name === 'EntityReference';
        }
        _isEntityCollectionReference(value) {
            return value && value.constructor && value.constructor.name === 'EntityCollectionReference';
        }
        async _getEntityByReference(value, context, options) {
            const entityReference = value;
            const entityType = await (await this.getDatastoreService()).getEntityType(entityReference.type);
            const entity = await entityType.getById(entityReference.id, context, options);
            return entity;
        }
        async _getEntityCollectionByReference(entityCollectionTypeKey, context, options) {
            const entityCollectionType = await (await this.getDatastoreService()).getEntityType(entityCollectionTypeKey);
            const coll = await entityCollectionType.query(context, options);
            return coll;
        }
    }
    exports.Property = Property;
});

//# sourceMappingURL=property.js.map
