"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_contracts_1 = require("@essential-projects/core_contracts");
const metadata_1 = require("@essential-projects/metadata");
const index_1 = require("./index");
const toposort = require("toposort");
const uuidModule = require("uuid");
const uuid = uuidModule;
class Entity {
    constructor(entityDependencyHelper, context, schema, propertyBag, entityType) {
        this._entityDependencyHelper = undefined;
        this._propertyBag = undefined;
        this._derivedClassInstance = undefined;
        this._context = undefined;
        this._schema = undefined;
        this._options = undefined;
        this._entityType = undefined;
        this._entityDependencyHelper = entityDependencyHelper;
        this._propertyBag = propertyBag;
        this._context = context;
        this._schema = schema;
        this._entityType = entityType;
    }
    get context() {
        return this._context;
    }
    get schema() {
        return this._schema;
    }
    get invoker() {
        return this._entityDependencyHelper.invoker;
    }
    get entityType() {
        return this._entityType;
    }
    get securityService() {
        return this._entityDependencyHelper.securityService;
    }
    get propertyBag() {
        return this._propertyBag;
    }
    async getDatastoreService() {
        return this._entityDependencyHelper.getDatastoreService();
    }
    get isNew() {
        return this.propertyBag.isNew;
    }
    get isDirty() {
        return this.propertyBag.isDirty();
    }
    get derivedClassInstance() {
        return this._derivedClassInstance;
    }
    async initialize(derivedClassInstance) {
        if (!derivedClassInstance) {
            throw new Error('derived class instance is missing');
        }
        this._derivedClassInstance = derivedClassInstance;
        this._propertyBag = await this.createPropertyBag(this.schema);
    }
    async createPropertyBag(schema) {
        const propertyBag = this.propertyBag;
        const attributeKeys = Object.keys(schema.attributes);
        attributeKeys.forEach((attributeKey) => {
            const attribute = schema.attributes[attributeKey];
            if (attribute.calculated) {
                const getCallback = attribute.calculated.onGet;
                const setCallback = attribute.calculated.onSet;
                propertyBag.createCalculatedProperty(attribute, attributeKey, getCallback, setCallback);
            }
            else {
                const onInitCallback = attribute.onInit;
                propertyBag.createProperty(attribute, attributeKey, onInitCallback);
            }
        });
        await propertyBag.setupPropertyBag();
        return propertyBag;
    }
    get id() {
        return this.getProperty(this, 'id');
    }
    set id(value) {
        this.setProperty(this, 'id', value);
    }
    getProperty(propertyThisContext, propertyName) {
        return this.propertyBag.getProperty(propertyThisContext, propertyName);
    }
    async getPropertyLazy(propertyThisContext, propertyName, context, options) {
        return this.propertyBag.getPropertyLazy(propertyThisContext, propertyName, context, options);
    }
    setProperty(propertyThisContext, propertyName, value) {
        const propertySchema = this.schema.attributes ? this.schema.attributes[propertyName] : undefined;
        const propertyIsValid = propertySchema !== undefined && value !== undefined;
        if (!propertyIsValid) {
            return this.propertyBag.setProperty(propertyThisContext, propertyName, value);
        }
        const isPojo = propertySchema.isComplexType && !this._isResolvableEntity(value);
        if (!isPojo) {
            return this.propertyBag.setProperty(propertyThisContext, propertyName, value);
        }
        const valueIsValidObject = typeof value === 'object' && value.id !== undefined;
        const valueIsString = typeof value === 'string';
        const canResolvePropertyToEntity = valueIsValidObject || valueIsString;
        if (!canResolvePropertyToEntity) {
            throw new Error(`unable to construct a resolvable entity representation for property ${propertyName} with data ${JSON.stringify(value)}!`);
        }
        let resolvedValue;
        if (valueIsValidObject) {
            resolvedValue = new index_1.EntityReference(this.schema.namespace, propertySchema.type, value.id);
        }
        else if (valueIsString) {
            resolvedValue = new index_1.EntityReference(this.schema.namespace, propertySchema.type, value);
        }
        return this.propertyBag.setProperty(propertyThisContext, propertyName, resolvedValue);
    }
    getCalculatedProperty(propertyThisContext, propertyName, context) {
        return this.propertyBag.getCalculatedProperty(propertyThisContext, propertyName, context);
    }
    setCalculatedProperty(propertyThisContext, propertyName, value, context) {
        return this.propertyBag.setCalculatedProperty(propertyThisContext, propertyName, value, context);
    }
    async toPojo(context, options) {
        const persistanceMode = options && options.mode && options.mode === core_contracts_1.SaveMode.persistance;
        const select = options && options.select ? options.select : undefined;
        options = options || {};
        if (!options.hasOwnProperty('_currentDepth')) {
            options._currentDepth = 0;
        }
        if (persistanceMode) {
            return this._toPojoForPersistance(context, options);
        }
        const currentDepth = options._currentDepth;
        const maxDepth = options.hasOwnProperty('maxDepth') ? options.maxDepth : 10;
        const datastoreService = await this.getDatastoreService();
        const sealInternals = (datastoreService && datastoreService.config && datastoreService.config.sealInternals);
        const hideInternals = sealInternals || (options && options.hideInternals);
        let data;
        if (maxDepth === undefined || currentDepth <= maxDepth) {
            let propertyKeysToExport = this.propertyBag.propertyKeys;
            if (select) {
                propertyKeysToExport = this._filterPropertyKeysBySelect(propertyKeysToExport, select);
            }
            if (hideInternals) {
                propertyKeysToExport = this._filterPropertyKeysByInternal(propertyKeysToExport);
            }
            data = await this.propertyBag.toPojo(this.derivedClassInstance, context, options, propertyKeysToExport);
            if (!hideInternals) {
                const meta = {
                    type: this.entityType.type,
                    namespace: this.entityType.namespace,
                    isRef: false,
                    isNew: this.isNew,
                };
                data._meta = meta;
            }
        }
        else {
            data = this.getEntityReference().toPojo();
        }
        return data;
    }
    async _toPojoForPersistance(context, options) {
        const saveFullEntity = options && options.saveFullEntity;
        const propertyKeysToExport = saveFullEntity ? this.propertyBag.propertyKeys : this.propertyBag.dirtyPropertyKeys;
        options = options || {};
        options.maxDepth = 0;
        const data = await this.propertyBag.toPojo(this.derivedClassInstance, context, options, propertyKeysToExport);
        await this._encryptData(data);
        return data;
    }
    _filterPropertyKeysBySelect(propertyKeys, select) {
        return select.filter((propertyKey) => {
            return (propertyKeys.indexOf(propertyKey) !== -1);
        });
    }
    _filterPropertyKeysByInternal(propertyKeys) {
        return propertyKeys.filter((propertyKey) => {
            const attributeSchema = this.schema.attributes[propertyKey];
            return attributeSchema && !attributeSchema.isInternal;
        });
    }
    async fromPojo(context, data, options) {
        if (!data) {
            return;
        }
        const dataComesFromAdapter = options && options.mode && options.mode === core_contracts_1.SaveMode.persistance;
        const propertyKeys = Object.keys(data);
        await this._resolveEntityReferences(data, propertyKeys);
        if (dataComesFromAdapter) {
            await this._fromRemotePojo(context, data, propertyKeys);
            await this._resolveEntityInstances(data, propertyKeys, options);
            await this._resolveEntityCollections(context, propertyKeys, options);
            await this.calculate(context, options);
        }
        else {
            this._filterAdapterMetadata(data, propertyKeys);
            await this._fromLocalPojo(context, data, propertyKeys);
        }
    }
    async _resolveEntityCollections(context, propertyKeys, options) {
        for (const propertyKey of propertyKeys) {
            const property = this.propertyBag.properties[propertyKey];
            if (property.isEntityCollectionReference()) {
                const ref = property.getValue(null);
                if (ref && ref.resolve) {
                    const queryOptions = {
                        query: {
                            attribute: ref.relatedAttribute,
                            operator: '=',
                            value: ref.id,
                        },
                    };
                    const expandOption = options.expandCollection || options.expandEntity;
                    if (expandOption) {
                        const foundExpands = expandOption.filter((expand) => propertyKey === expand.attribute);
                        if (foundExpands.length === 1) {
                            const expand = foundExpands[0];
                            if (expand.childAttributes && expand.childAttributes.length > 0) {
                                queryOptions.select = expand.childAttributes.map((child) => child.attribute);
                            }
                            queryOptions.expandCollection = expand.childAttributes;
                        }
                    }
                    const dataStoreService = await this.getDatastoreService();
                    const entityType = await dataStoreService.getEntityType(ref.relatedType);
                    const coll = await entityType.query(context, queryOptions);
                    this.propertyBag.setProperty(this, propertyKey, coll);
                }
            }
        }
    }
    async calculate(context, options) {
        const select = options && options.select ? options.select : undefined;
        let propertyKeys = this.propertyBag.propertyKeys;
        if (select) {
            propertyKeys = this._filterPropertyKeysBySelect(propertyKeys, select);
        }
        const topSortEdges = [];
        for (const propertyKey in propertyKeys) {
            const property = this.propertyBag.properties[propertyKeys[propertyKey]];
            if (property.schema && property.schema.calculated) {
                if (property.schema.expandEntity) {
                    property.schema.expandEntity.forEach((expand) => {
                        topSortEdges.push([propertyKeys[propertyKey], expand.attribute]);
                    });
                }
                else {
                    topSortEdges.push([propertyKeys[propertyKey], 'root']);
                }
            }
        }
        const topSortResult = toposort(topSortEdges).reverse().filter((edge) => edge !== 'root');
        for (const propertyKey in topSortResult) {
            const property = this.propertyBag.properties[topSortResult[propertyKey]];
            if (property.schema && property.schema.calculated) {
                await property.getValueLazy(this, context);
            }
        }
    }
    _isEntityReference(value) {
        return value && value.constructor && value.constructor.name === 'EntityReference';
    }
    _isEntityCollectionReference(value) {
        return value && value.constructor && value.constructor.name === 'EntityCollectionReference';
    }
    _isReference(value) {
        return (this._isEntityCollectionReference(value) || this._isEntityReference(value));
    }
    _isResolvableEntity(value) {
        return this._isEntityReference(value) || value.getEntityReference !== undefined
            && value.entityType !== undefined
            && value.propertyBag !== undefined;
    }
    _isResolvableEntityCollection(value) {
        return this._isEntityCollectionReference(value) || value && value.constructor && value.constructor.name === 'EntityCollection';
    }
    _isResolvable(value) {
        return (this._isResolvableEntity(value) || this._isResolvableEntityCollection(value));
    }
    _filterAdapterMetadata(data, propertyKeys) {
        if (propertyKeys.indexOf('_meta') !== -1) {
            propertyKeys.splice(propertyKeys.indexOf('_meta'), 1);
            delete data._meta;
        }
    }
    async _resolveEntityInstances(data, propertyKeys, options) {
        const resolvePromises = propertyKeys.map(async (propertyKey) => {
            const propertyValue = data[propertyKey];
            const propertySchema = this.schema.attributes ? this.schema.attributes[propertyKey] : undefined;
            if (propertySchema && propertySchema.isComplexType) {
                if (typeof propertyValue === 'undefined' || propertyValue === null || this._isReference(propertyValue)) {
                    return;
                }
                const entityType = await (await this.getDatastoreService()).getEntityType(propertySchema.type);
                const createEntityOption = Object.assign({}, options);
                if (createEntityOption.expandEntity) {
                    const foundExpands = createEntityOption.expandEntity.filter((expand) => propertyKey === expand.attribute);
                    if (foundExpands.length === 1) {
                        const expand = foundExpands[0];
                        if (expand.childAttributes && expand.childAttributes.length > 0) {
                            createEntityOption.select = expand.childAttributes.map((child) => child.attribute);
                        }
                        createEntityOption.expandEntity = expand.childAttributes;
                    }
                }
                const entity = await entityType.createEntity(this.context, propertyValue, createEntityOption);
                data[propertyKey] = entity;
                this.propertyBag.setProperty(this, propertyKey, entity);
            }
        });
        await Promise.all(resolvePromises);
    }
    async _resolveEntityReferences(data, propertyKeys) {
        const resolvePromises = propertyKeys.map(async (propertyKey) => {
            const propertyValue = data[propertyKey];
            const propertySchema = this.schema.attributes ? this.schema.attributes[propertyKey] : undefined;
            const valueType = typeof propertyValue;
            if (valueType === 'undefined' || propertyValue === null) {
                return;
            }
            if (propertySchema && propertySchema.isComplexType) {
                if (propertySchema.isList) {
                    const collectionReference = new index_1.EntityCollectionReference(this.schema.namespace, propertySchema.type, propertyValue.id, propertySchema.type, propertySchema.relatedAttribute, propertyValue.resolve);
                    data[propertyKey] = collectionReference;
                }
                else if (valueType === 'string') {
                    const entityReference = new index_1.EntityReference(this.schema.namespace, propertySchema.type, propertyValue);
                    data[propertyKey] = entityReference;
                }
                else {
                    if (typeof propertyValue === 'object' && propertyValue._meta && propertyValue._meta.isRef && propertyValue.id) {
                        const entityReference = new index_1.EntityReference(this.schema.namespace, propertySchema.type, propertyValue.id);
                        data[propertyKey] = entityReference;
                    }
                }
            }
        });
        await Promise.all(resolvePromises);
    }
    async _fromRemotePojo(context, data, propertyKeys) {
        await this._decryptData(data);
        this.propertyBag.updateFromRemote(data);
    }
    async _fromLocalPojo(context, data, propertyKeys) {
        this.propertyBag.updateFromLocal(data, this);
    }
    async _encryptData(data) {
        const keys = Object.keys(data);
        const encryptPromises = keys.map(async (propertyKey) => {
            const attributeSchema = this.schema.attributes[propertyKey];
            const value = data[propertyKey];
            if (attributeSchema.encryptionType) {
                const encryptOptions = { encryptionType: attributeSchema.encryptionType };
                const encryptedValue = await this.securityService.encrypt(value, encryptOptions);
                data[propertyKey] = encryptedValue;
            }
        });
        await Promise.all(encryptPromises);
    }
    async _decryptData(data) {
        const keys = Object.keys(data);
        const decryptPromises = keys.map(async (propertyKey) => {
            const attributeSchema = this.schema.attributes[propertyKey];
            const value = data[propertyKey];
            if (attributeSchema.encryptionType) {
                const decryptOptions = { encryptionType: attributeSchema.encryptionType };
                const decryptedValue = await this.securityService.decrypt(value, decryptOptions);
                data[propertyKey] = decryptedValue;
            }
        });
        await Promise.all(decryptPromises);
    }
    async secureCompare(attributeName, valueToCompare) {
        const attributeSchema = this.schema.attributes[attributeName];
        const value = this[attributeName];
        const decryptOptions = { encryptionType: attributeSchema.encryptionType };
        return await this.securityService.secureCompare(value, valueToCompare, decryptOptions);
    }
    async save(context, options) {
        const reloadAfterSave = options && options.hasOwnProperty('reloadAfterSave') ? options.reloadAfterSave : true;
        const savedEntity = await this.entityType.save(this, context, options);
        if (!reloadAfterSave) {
            this.propertyBag.isNew = false;
        }
        return savedEntity;
    }
    async remove(context, options) {
        return await this.entityType.remove(this, context, options);
    }
    getEntityReference() {
        return new index_1.EntityReference(this.schema.namespace, this.entityType.type, this.id);
    }
}
__decorate([
    metadata_1.schemaAttribute({
        type: core_contracts_1.SchemaAttributeType.string,
        isPrimaryKey: true,
        onInit: () => {
            return uuid.v4();
        },
    })
], Entity.prototype, "id", null);
exports.Entity = Entity;

//# sourceMappingURL=entity.js.map
