import { ExecutionContext, IEntity, IEntityReference, IFromPojoOptions, IInheritedSchema, IPrivateQueryOptions, IPrivateRemoveOptions, IPrivateSaveOptions, IToPojoOptions } from '@essential-projects/core_contracts';
import { IInvoker } from '@essential-projects/invocation_contracts';
import { ISecurityService } from '@essential-projects/security_service_contracts';
import { IDatastoreService, IEntityDependencyHelper, IEntityType, IPropertyBag } from './index';
export declare class Entity implements IEntity {
    private _entityDependencyHelper;
    private _propertyBag;
    private _derivedClassInstance;
    private _context;
    private _schema;
    private _options;
    private _entityType;
    constructor(entityDependencyHelper: IEntityDependencyHelper, context: ExecutionContext, schema: IInheritedSchema, propertyBag: IPropertyBag, entityType: IEntityType<IEntity>);
    readonly context: ExecutionContext;
    readonly schema: IInheritedSchema;
    protected readonly invoker: IInvoker;
    protected readonly entityType: IEntityType<IEntity>;
    protected readonly securityService: ISecurityService;
    protected readonly propertyBag: IPropertyBag;
    protected getDatastoreService(): Promise<IDatastoreService>;
    readonly isNew: boolean;
    readonly isDirty: boolean;
    private readonly derivedClassInstance;
    initialize(derivedClassInstance: IEntity): Promise<void>;
    private createPropertyBag;
    id: string;
    getProperty(propertyThisContext: any, propertyName: string): any;
    getPropertyLazy(propertyThisContext: any, propertyName: string, context: ExecutionContext, options?: IPrivateQueryOptions): Promise<any>;
    setProperty(propertyThisContext: any, propertyName: string, value: any): void;
    getCalculatedProperty(propertyThisContext: any, propertyName: string, context: ExecutionContext): Promise<any>;
    setCalculatedProperty(propertyThisContext: any, propertyName: string, value: any, context: ExecutionContext): Promise<void>;
    toPojo(context: ExecutionContext, options?: IToPojoOptions): Promise<any>;
    private _toPojoForPersistance;
    private _filterPropertyKeysBySelect;
    private _filterPropertyKeysByInternal;
    fromPojo(context: ExecutionContext, data: any, options?: IFromPojoOptions): Promise<void>;
    private _resolveEntityCollections;
    calculate(context: ExecutionContext, options?: IFromPojoOptions): Promise<void>;
    private _isEntityReference;
    private _isEntityCollectionReference;
    private _isReference;
    private _isResolvableEntity;
    private _isResolvableEntityCollection;
    private _isResolvable;
    private _filterAdapterMetadata;
    private _resolveEntityInstances;
    private _resolveEntityReferences;
    private _fromRemotePojo;
    private _fromLocalPojo;
    private _encryptData;
    private _decryptData;
    secureCompare(attributeName: string, valueToCompare: string): Promise<boolean>;
    save(context: ExecutionContext, options?: IPrivateSaveOptions): Promise<IEntity>;
    remove(context: ExecutionContext, options?: IPrivateRemoveOptions): Promise<boolean>;
    getEntityReference(): IEntityReference;
}
