import {IDataSource} from '../data_source';
import {IEntityCollection} from './ientity_collection';
import {ExpandType} from '../constants';

import {
  EntityTypeKey,
  ExecutionContext,
  IEntity,
  IExpand,
  IFromPojoOptions,
  IPrivateGetOptions,
  IPrivateQueryOptions,
  IPrivateRemoveOptions,
  IPrivateSaveOptions,
  TypeKey,
} from '@essential-projects/core_contracts';

export interface IEntityType<T extends IEntity> {
  name: EntityTypeKey;
  origin: string;
  namespace: string;
  dataSource: IDataSource<T>;
  type: TypeKey;
  query(context: ExecutionContext, options?: IPrivateQueryOptions): Promise<IEntityCollection<T>>;
  save(data: T, context: ExecutionContext, options?: IPrivateSaveOptions): Promise<T>;
  remove(data: T, context: ExecutionContext, options?: IPrivateRemoveOptions): Promise<boolean>;
  getById(id: string, context: ExecutionContext, options?: IPrivateGetOptions): Promise<T>;
  load(entity: T, context: ExecutionContext, options?: IPrivateGetOptions): Promise<void>;
  checkConnection(): Promise<boolean>;
  initialize(): Promise<void>;
  dispose(): Promise<void>;
  findOne(context: ExecutionContext, options?: IPrivateQueryOptions): Promise<T>;
  all(context: ExecutionContext, options?: IPrivateQueryOptions): Promise<IEntityCollection<T>>;
  count(context: ExecutionContext, options?: IPrivateQueryOptions): Promise<number>;
  createEntity(context: ExecutionContext, data?: any, options?: IFromPojoOptions): Promise<T>;
  getDefaultExpandOptions(expandType: ExpandType): Array<IExpand>;
  createEntityCollection(context: ExecutionContext,
                         data?: Array<any>,
                         totalCount?: number,
                         offset?: number,
                         limit?: number,
                         options?: IPrivateQueryOptions): Promise<IEntityCollection<T>>;
}
