import {ExecutionContext, ISchemaAttribute} from '@essential-projects/core_contracts';
import {IFactoryAsync} from 'addict-ioc';
import {ICalculatedProperty, IDatastoreService} from './../index';

export class CalculatedProperty implements ICalculatedProperty {

  private _schema: ISchemaAttribute = undefined;
  private _datastoreServiceFactory: IFactoryAsync<IDatastoreService> = undefined;
  private _datastoreService: IDatastoreService = undefined;
  private _context: ExecutionContext = undefined;
  private _remoteValue: any = undefined;
  private _propertyName: string = undefined;
  private _getCallback: Function = undefined;
  private _setCallback: Function = undefined;
  private _value: any = undefined;

  constructor(schema: ISchemaAttribute,
              datastoreServiceFactory: IFactoryAsync<IDatastoreService>,
              context: ExecutionContext,
              propertyName: string,
              getCallback: Function,
              setCallback: Function) {
    this._schema = schema;
    this._datastoreServiceFactory = datastoreServiceFactory;
    this._context = context;
    this._propertyName = propertyName;
    this._getCallback = getCallback;
    this._setCallback = setCallback;
  }

  public get schema(): ISchemaAttribute {
    return this._schema;
  }

  private async getDatastoreService(): Promise<IDatastoreService> {
    if (!this._datastoreService) {
      this._datastoreService = await this._datastoreServiceFactory();
    }

    return this._datastoreService;
  }

  private get context(): ExecutionContext {
    return this._context;
  }

  private get remoteValue(): any {
    return this._remoteValue;
  }

  public get propertyName(): string {
    return this._propertyName;
  }

  private get getCallback(): Function {
    return this._getCallback;
  }

  private get setCallback(): Function {
    return this._setCallback;
  }

  private get value(): any {
    return this._value;
  }

  public async initializeValue(): Promise<void> { return; }

  public getValue(valueSource: any): any {
    return this._getLastCalculatedValue();
  }

  public async getValueLazy(valueSource: any, context: ExecutionContext): Promise<any> {
    if (!this.getCallback) {
      throw new Error(`no getter function defined for calculated property "${this.propertyName}"`);
    }

    const getFunction: Function = this.getCallback;

    const calculatedValue: any = await getFunction.bind(valueSource)(context);
    this._value = calculatedValue;

    return this.value;
  }

  public setValue(valueTarget: any, value: any): void {
    this.setValueLazy(valueTarget, value, undefined);
  }

  public async setValueLazy(valueTarget: any, value: any, context: ExecutionContext): Promise<void> {
    if (this.setCallback) {
      const setFunction: Function = this.setCallback;

      await setFunction.bind(valueTarget)(context, value);
      this._value = value;
    }
  }

  public updateFromRemote(remoteValue: any): void { return; }

  public isDirty(): boolean {
    return false;
  }

  public isEntity(): boolean {
    return false;
  }

  public isEntityReference(): boolean {
    return false;
  }

  public isEntityCollectionReference(): boolean {
    return false;
  }

  private _getLastCalculatedValue(): any {
    return this.value;
  }
}
